@@ -33,10 +33,20 @@
 """
 git vcs support.
 
-New in ROS C-Turtle.
-
 refnames in git can be branchnames, hashes, partial hashes, tags. On
 checkout, git will disambiguate by checking them in that order, taking the first that applies
+
+This class aims to provide git for linear centralized workflows.
+This means we assume that the only relevant remote is the one named "origin",
+and we assume that commits once on origin remain on origin.
+
+A challenge with git is that it has strong reasonable
+conventions, but is very allowing for breaking them. E.g. it is
+possible to name remotes and branches with names like
+"refs/heads/master", give branches and tags the same name, or a valid
+SHA-ID as name, etc.  Similarly git allows plenty of ways to reference
+any object, in cae of ambiguities, git attempts to take the most
+reasonable disambiguation, and in some cases warns.
 """
 
 import subprocess
@@ -85,7 +95,7 @@ class GitClient(VcsClientBase):
             return False
         
         #since we cannot know whether refname names a branch, clone master initially
-        cmd = "git clone %s %s"%(url, self._path)
+        cmd = "git clone --recursive %s %s"%(url, self._path)
         if not subprocess.call(cmd, shell=True) == 0:
             return False
 
@@ -107,13 +117,16 @@ class GitClient(VcsClientBase):
         """interprets refname as a local branch, remote branch, tagname, hash, etc.
         If it is a branch, attempts to move to it unless already on it, and to fast-forward, unless not a tracking branch.
         Else go untracked on tag or whatever refname is. Does not leave if current commit would become dangling."""
+        # try calling git fetch just once per call to update()
+        need_to_fetch = True
         if not self.detect_presence():
             return False
 
         # are we on any branch?
         current_branch = self.get_branch()
         if current_branch:
-            branch_parent = self.get_branch_parent()
+            branch_parent = self.get_branch_parent(need_to_fetch)
+            need_to_fetch = False
         else:
             branch_parent = None
 
@@ -129,18 +142,18 @@ class GitClient(VcsClientBase):
         # if same_branch and branch_parent == None:
         #   already on branch, nothing to pull as non-tracking branch
         if same_branch and branch_parent != None:
-            if not self._do_fast_forward():
+            if not self._do_fast_forward(need_to_fetch):
                 return False
+            need_to_fetch = False
         elif not same_branch:
             # refname can be a different branch or something else than a branch
-            need_to_fetch = True
+            
             refname_is_local_branch = self.is_local_branch(refname)
             if refname_is_local_branch == True:
                 # might also be remote branch, but we treat it as local
                 refname_is_remote_branch = False
             else:
                 refname_is_remote_branch = self.is_remote_branch(refname, fetch = need_to_fetch)
-                # one fetch is enough
                 need_to_fetch = False
             refname_is_branch = refname_is_remote_branch or refname_is_local_branch
 
@@ -165,9 +178,9 @@ class GitClient(VcsClientBase):
             
             if refname_is_local_branch:
                 # if we just switched to a local tracking branch (not created one), we should also fast forward
-                new_branch_parent = self.get_branch_parent()
+                new_branch_parent = self.get_branch_parent(fetch = need_to_fetch)
                 if new_branch_parent != None:
-                    if not self._do_fast_forward():
+                    if not self._do_fast_forward(fetch = need_to_fetch):
                         return False
             
         return self.update_submodules()
@@ -240,10 +253,10 @@ class GitClient(VcsClientBase):
                 return False
             output = subprocess.Popen(['git branch -r'], shell=True, cwd= self._path, stdout=subprocess.PIPE).communicate()[0]
             for l in output.splitlines():
-                elems = l.split()
-                if len(elems) == 1:
-                    br_names = elems[0].split('/')
-                    if len(br_names) == 2 and br_names[0] == 'origin' and br_names[1] == branch_name:
+                elem = l.split()[0]
+                rem_name = elem[:elem.find('/')]
+                br_name = elem[elem.find('/') + 1:]
+                if rem_name == "origin" and br_name == branch_name:
                         return True
         return False
 
@@ -272,20 +285,40 @@ class GitClient(VcsClientBase):
         return None
 
 
-    def get_branch_parent(self):
+    def get_branch_parent(self, fetch=False):
         """return the name of the branch this branch tracks, if any"""
         if self.path_exists():
-            output = subprocess.Popen(['git config --get branch.%s.merge'%self.get_branch()], shell=True, cwd= self._path, stdout=subprocess.PIPE).communicate()[0].strip()
+            # get name of configured merge ref.
+            output = subprocess.Popen(['git config --get-all branch.%s.merge'%self.get_branch()], shell=True, cwd= self._path, stdout=subprocess.PIPE).communicate()[0].strip()
             if not output:
-                print "No output of get branch.%s.merge"%self.get_branch()
                 return None
-            elems = output.split('/')
-            # due to an earlier bug vcstools would set up to track a tag
-            if len(elems) != 3 or elems[0] != 'refs' or (elems[1] != 'heads' and elems[1] != 'tags'):
-                print "elems improperly formatted", elems
-            else:
-                if elems[1] == 'heads':
-                    return elems[2]
+            lines = output.splitlines()
+            if len(lines) > 1:
+                print "vcstools unable to handle multiple merge references for branch %s:\n%s"%(self.get_branch(), output)
+                return None
+            # get name of configured remote
+            output2 = subprocess.Popen(['git config --get-all branch.%s.remote'%self.get_branch()], shell=True, cwd= self._path, stdout=subprocess.PIPE).communicate()[0].strip()
+            if output2 != "origin":
+                print "vcstools only handles branches tracking remote 'origin', branch '%s' tracks remote '%s'"%(self.get_branch(), output2)
+                return None
+            output = lines[0]
+            # output is either refname, or /refs/heads/refname, or heads/refname
+            # we would like to return refname
+            # however, user could also have named any branch "/refs/heads/refname", for some unholy reason
+            # check all known branches on remote for refname, then for the odd cases, as git seems to do
+            candidate = output
+            if candidate.startswith('refs/'):
+                candidate = candidate[len('refs/'):]
+            if candidate.startswith('heads/'):
+                candidate = candidate[len('heads/'):]
+            elif candidate.startswith('tags/'):
+                candidate = candidate[len('tags/'):]
+            elif candidate.startswith('remotes/'):
+                candidate = candidate[len('remotes/'):]
+            if self.is_remote_branch(candidate, fetch=fetch):
+                return candidate
+            if output != candidate and self.is_remote_branch(output, fetch=False):
+                return output
         return None
 
 
@@ -381,22 +414,20 @@ class GitClient(VcsClientBase):
         """Execute git fetch if necessary, and if we can fast-foward,
         do so to the last fetched version using git rebase. Returns
         False on command line failures"""
-        # the safe way would be to git pull, and if that caused a merge or merge conflict, abort.
-        # That would mean after the pull, we check whether we are in conflict or ahead of remote (merge commit created locally)
-        parent = self.get_branch_parent()
-        if parent != None and self.rev_list_contains("remotes/origin/%s"%parent, self.get_version(), fetch = fetch):
+        parent = self.get_branch_parent(fetch = fetch)
+        if parent != None and self.rev_list_contains("remotes/origin/%s"%parent, self.get_version(), fetch = False):
             # Rebase, do not pull, because somebody could have
             # commited in the meantime.
             if LooseVersion(self.gitversion) >= LooseVersion('1.7.1'):
                 # --keep allows o rebase even with local changes, as long as
                 # local changes are not in files that change between versions
-                cmd = "git reset --keep remotes/origin/%s"%self.get_branch_parent()
+                cmd = "git reset --keep remotes/origin/%s"%parent
                 if subprocess.call(cmd, cwd=self._path, shell=True) == 0:
                     return True
             else:
                 # prior to version 1.7.1, git does not know --keep
                 # Do not merge, rebase does nothing when there are local changes
-                cmd = "git rebase remotes/origin/%s"%self.get_branch_parent()
+                cmd = "git rebase remotes/origin/%s"%parent
                 if subprocess.call(cmd, cwd=self._path, shell=True) == 0:
                     return True
             return False
